using System;
using System.Collections.Generic;
using System.Text;
using System.Data.SqlClient;
using System.Collections;
using System.Data;


[assembly: log4net.Config.XmlConfigurator(Watch = true)]
namespace ClassificationApplication.ArticleStore.DBStore
{
    public class ArticleFetcher
    {

        private SqlConnection conn;
        private static readonly log4net.ILog log = log4net.LogManager.GetLogger(System.Reflection.MethodBase.GetCurrentMethod().DeclaringType);
        private static readonly ArticleFetcher fetcher = new ArticleFetcher();
        private Dictionary<string, DateTime> dateDictionary;

        public ArticleFetcher()
        {
            log.Info("Initializing ArticleStore");
            dateDictionary = new Dictionary<string,DateTime>();
            conn = new SqlConnection();
            conn.ConnectionString = "Data Source=tcp:129.241.102.186,1500;Initial Catalog=newsdatabase;User ID=databaseuser;Password=databasebruker1;";
            
        }

        public Article[] FetchArticles(DateTime startDate, DateTime endDate)
        {
            string[] urls = FetchUrlsFromDB(startDate, endDate);
            Console.WriteLine("Url'er funnet: " + urls.Length);
            Article[] articles = FetchArticlesFromDB(urls);
            Console.WriteLine("Artikler funnet: " + articles.Length);
            return articles;
        }

        private Article FetchArticleFromDB(string url)
        {
            conn.Open();
            string[] paragraphs = FetchParagraphsFromDB(url);

            SqlCommand selectCommand;
            SqlDataAdapter articleAdapter;
            DataSet paragraphsSet = new DataSet();
            
            string selectString = "select title, ingress from newsArticle " +
            "where url like '" + url + "';";

            selectCommand = new SqlCommand(selectString, conn);
            articleAdapter = new SqlDataAdapter(selectCommand);

            //conn.Open();
            articleAdapter.Fill(paragraphsSet);

            DataTable dataTable = paragraphsSet.Tables[0];

            if (dataTable.Rows.Count == 0)
            {
                conn.Close();
                throw new ArticleNotFoundInDBException();
            }
            else
            {
                string title = dataTable.Rows[0][0].ToString();
                string ingress = dataTable.Rows[0][1].ToString();
                conn.Close();

                return new Article(url, title, ingress, paragraphs, dateDictionary[url]);
            }
        }
        private Article[] FetchArticlesFromDB(string[] urls)
        {
            ArrayList articles = new ArrayList();
            try
            {
                conn.Open();
                string[] paragraphs;
                SqlCommand sqlCommand = new SqlCommand();
                sqlCommand.Connection = conn;
                SqlDataAdapter articleAdapter = new SqlDataAdapter();
                DataSet articleSet;
                DataTable dataTable;
                string title;
                string ingress;

                foreach (string url in urls)
                {
                    //Console.WriteLine(url);
                    articleSet = new DataSet();
                    paragraphs = FetchParagraphsFromDB(url);
                    for (int i = 0; i < paragraphs.Length; i++)
                    {
                        string paragraph = paragraphs[i];

                        while (paragraph.Contains("^"))
                        {
                            paragraph = paragraph.Replace("^", "");
                        }
                        while (paragraph.Contains("\n"))
                        {
                            paragraph = paragraph.Replace("\n", "");
                        }
                        while (paragraph.Contains("  "))
                        {
                            paragraph = paragraph.Replace("  ", " ");
                        }
                        paragraphs[i] = paragraph;
                    }
                    sqlCommand.CommandText = "select title, ingress from newsArticle " +
                        "where url like '" + url + "';";
                    articleAdapter.SelectCommand = sqlCommand;
                    articleAdapter.Fill(articleSet);

                    dataTable = articleSet.Tables[0];

                    if (dataTable.Rows.Count > 0)
                    {
                        title = dataTable.Rows[0][0].ToString().Trim();
                        while (title.Contains("^"))
                        {
                            title = title.Replace("^", "");
                        }
                        while (title.Contains("\n"))
                        {
                            title = title.Replace("\n", "");
                        }
                        while (title.Contains("  "))
                        {
                            title = title.Replace("  ", " ");
                        }
                        ingress = dataTable.Rows[0][1].ToString();
                        while (ingress.Contains("^"))
                        {
                            ingress = ingress.Replace("^", "");
                        }
                        while (ingress.Contains("\n"))
                        {
                            ingress = ingress.Replace("\n", "");
                        }
                        while (ingress.Contains("  "))
                        {
                            ingress = ingress.Replace("  ", " ");
                        }
                        articles.Add(new Article(url, title, ingress, paragraphs, dateDictionary[url]));
                    }
                }
            }
            catch (System.InvalidOperationException e)
            {
                log.Error(e);
            }
            catch (System.Data.SqlClient.SqlException e)
            {
                log.Error(e);
            }
            finally
            {
                conn.Close();
            }
            return (Article[])articles.ToArray(typeof(Article));
        }
        private string[] FetchParagraphsFromDB(string url)
        {
            SqlCommand selectCommand;
            SqlDataAdapter paragraphsAdapter;
            DataSet paragraphsSet = new DataSet();
            ArrayList paragraphList = new ArrayList();

            string selectString = "select text from newsParagraph " +
            "where url like '" + url + "' " +
            "order by p_id asc;";

            selectCommand = new SqlCommand(selectString, conn);
            paragraphsAdapter = new SqlDataAdapter(selectCommand);

            //conn.Open();
            paragraphsAdapter.Fill(paragraphsSet);

            DataTable dataTable = paragraphsSet.Tables[0];

            foreach (DataRow row in dataTable.Rows)
            {
                paragraphList.Add(row[0].ToString());
            }
            //conn.Close();
            string[] paragraphArray = (String[])paragraphList.ToArray(typeof(String));
            return paragraphArray;
        }
        private string[] FetchUrlsFromDB(DateTime startTime, DateTime endTime)
        {
            
            SqlCommand selectCommand;
            SqlDataAdapter paragraphsAdapter;
            DataSet paragraphsSet = new DataSet();
            ArrayList urlList = new ArrayList();

            string selectString = "select url, date_stored from rssNews " +
            "where date_stored > '" + startTime + "' " +
            "and date_stored < '"+ endTime + "' ;";

            selectCommand = new SqlCommand(selectString, conn);
            paragraphsAdapter = new SqlDataAdapter(selectCommand);

            conn.Open();
            paragraphsAdapter.Fill(paragraphsSet);

            DataTable dataTable = paragraphsSet.Tables[0];

            foreach (DataRow row in dataTable.Rows)
            {
                urlList.Add(row[0].ToString());
                dateDictionary[row[0].ToString()] = (DateTime)row[1];//TODO: Sjekk at dette er riktig
            }
            conn.Close();
            string[] paragraphArray = (String[])urlList.ToArray(typeof(String));
                
            return paragraphArray;
        }
    }
}
