using System;
using System.Collections.Generic;
using System.Text;
using ClassificationApplication.ArticleStore.DBStore;
using Lucene.Net.Index;
using Lucene.Net.Analysis;
using Lucene.Net.Documents;
using System.IO;
using Tagger.Tagging;
using Tagger.Lemmatizing;

namespace ClassificationApplication.ArticleStore.Indexer
{
    internal class ArticleIndexer
    {
        public string IndexDirectory
        {
            get { return Properties.Settings.Default.IndexDirectory; }
        }
        public void IndexArticles(DateTime startDate, DateTime endDate)
        {
            NorwegianLemmatizer lemmatizer = new NorwegianLemmatizer();
            NorwegianPOSTagger tagger = new NorwegianPOSTagger(NorwegianPOSTagger.Lemmatize, true);            
            Console.WriteLine("Henter artikler");
            ArticleFetcher fetcher = new ArticleFetcher();
            Article[] articles = fetcher.FetchArticles(startDate, endDate);
            if (!Directory.Exists(IndexDirectory))
                Directory.CreateDirectory(IndexDirectory);
            IndexWriter writer = new IndexWriter(IndexDirectory, new SimpleAnalyzer(), true);
            int i = 0;
            foreach (Article article in articles)
            {
                try
                {
                    string paragraphText = "";

                    foreach (string paragraph in article.Paragraphs)
                    {
                        paragraphText = string.Concat(paragraphText.Trim(), " ", paragraph);
                    }

                    string titleTagged = tagger.Tag(article.Title.Trim());
                    Field titleField = new Field("title", LemmatizeField(titleTagged), true, true, true, true);
                    Field titleTaggedField = new Field("titleTagged", titleTagged, true, true, true, true);

                    string ingressTagged = tagger.Tag(article.Ingress.Trim());
                    Field ingressField = new Field("ingress", LemmatizeField(ingressTagged), true, true, true, true);
                    Field ingressTaggedField = new Field("ingressTagged", ingressTagged, true, false, false, false);
                    Field urlField = new Field("url", article.Url.Trim(), true, false, false, false);
                    
                    string paragraphTagged = tagger.Tag(paragraphText.Trim());
                    Field paragraphField = new Field("paragraph", LemmatizeField(paragraphTagged), true, true, true, true);
                    Field paragraphTaggedField = new Field("paragraphTagged", paragraphTagged, true, false, false, false);

                    Field dateField = new Field("date", DateTools.DateToString(article.Date, DateTools.Resolution.DAY), true, true, false);
                    //dateField = Field.Keyword("date", article.Date);

                    Document doc = new Document();
                    doc.Add(titleField);
                    doc.Add(titleTaggedField);
                    doc.Add(ingressField);
                    doc.Add(ingressTaggedField);
                    doc.Add(urlField);
                    //doc.Add(paragraphField);
                    //doc.Add(paragraphTaggedField);
                    doc.Add(dateField);

                    writer.AddDocument(doc);

                    i++;
                    Console.WriteLine(i);
                }
                catch (Lucene.Net.QueryParsers.ParseException pe)
                {
                    Console.WriteLine("Problemer med artikkel " + i);
                    Console.WriteLine(pe);
                }
                catch (ArgumentException ae)
                {
                    Console.WriteLine("Problemer med artikkel " + i);
                    Console.WriteLine(ae);
                }
            }
            Console.WriteLine("Done!");
            writer.Close();
        }
        private string LemmatizeField(string field)
        {
            string lemmatized = "";
            string []lemmaTagPair = field.Split(' ');
            foreach (string lemma in lemmaTagPair)
            {
                lemmatized = string.Concat(lemmatized, " ", lemma.Split('/')[0]);
            }
            return lemmatized.Trim();
        }
    }
}
