using System;
using System.Collections.Generic;
using System.Windows.Forms;
using System.IO;
using ClassificationApplication.ArticleStore.Indexer;
using ClassificationApplication.Search;
using Lucene.Net.Search;
using ClassificationApplication.FeatureExtractor;
using ClassificationApplication.FeatureExtractor.IndexComparator;
using ClassificationApplication.ArticleStore.DBStore;

namespace ClassificationApplication
{
    static class Program
    {
        /// <summary>
        /// The main entry point for the application.
        /// </summary>
        [STAThread]
        public static void Main(string[] args)
        {
            //StoreArticlesAsFiles(new DateTime(2006, 5, 14), new DateTime(2006, 6, 14));
            ArticleIndexer indexer = new ArticleIndexer();
            //indexer.IndexArticles(new DateTime(2006, 1, 1), new DateTime(2010, 1, 1));

            WriteSimilaritiesToFile();
            //FindSimilaritiesBetweenEnities();
        }
        private static void StoreHitsAsFile(Hits hits, string path)
        {
            using (StreamWriter stw = new StreamWriter(path, false, System.Text.Encoding.Default))
            {
                for (int i = 0; i < hits.Length(); i++)
                {
                    stw.WriteLine("Document: " + i);
                    stw.WriteLine(hits.Doc(i).GetField("title").StringValue());
                    stw.WriteLine(hits.Doc(i).GetField("ingress").StringValue() + "\n");
                }
            }
        }

        private static void StoreArticlesAsFiles(DateTime startDate, DateTime endDate)
        {
            //To store articles as files (used for corpus creation; last included dates: 1.1-14.5)
            string directory = Tagger.Tagging.Training.TaggerTrainer.ArticleDirectory;
            if (!Directory.Exists(directory))
                Directory.CreateDirectory(directory);
            Article[] articles;
            ArticleFetcher fetcher = new ArticleFetcher();
            articles = fetcher.FetchArticles(startDate, endDate);
            int i = 0;
            foreach (Article art in articles)
            {
                WriteArticleToFile(directory + "a_del" + i + ".txt", art);
                i++;
            }
        }
        static void WriteArticleToFile(string fileName, Article art)
        {
            using (StreamWriter writer = new StreamWriter(fileName, false))
            {
                Console.WriteLine(fileName);
                writer.Write(art);
            }
        }

        private static void FindSimilaritiesBetweenEnities()
        {
            string[] queryString1 = new string[] { "statoil" };
            string[] queryString2 = new string[] { "pgs" };
            DateTime startDate = new DateTime(2006, 2, 13);
            DateTime endDate = new DateTime(2006, 2, 20);
            ConceptSearcher searcher = new ConceptSearcher();
            //TODO: Fiks vekting: TF for dokument * antall dok
            Hits firstSearch = searcher.SearchForConcepts(queryString1, queryString2, startDate, endDate);
            
            startDate = new DateTime(2006, 5, 8);
            endDate = new DateTime(2006, 5, 15);
            Hits secondSearch = searcher.SearchForConcepts(queryString1, queryString2, startDate, endDate);

            Extractor e1 = new Extractor(firstSearch);
            Extractor e2 = new Extractor(secondSearch);

            double similarity = e1.GetIndex(Globals.PhraseRegex).Compare(e2.GetIndex(Globals.PhraseRegex));

        }
        private static void WriteSimilaritiesToFile()
        {
            string[] queryString = new string[] { "statoil" };
            DateTime startDate = new DateTime(2006, 2, 13);
            DateTime endDate = new DateTime(2006, 2, 20);
            ConceptSearcher searcher = new ConceptSearcher();
            //TODO: Fiks vekting: TF for dokument * antall dok
            Hits firstSearch = searcher.SearchForConcepts(queryString, startDate, endDate);
            Console.WriteLine("StartDate: " + startDate + "\n" + "EndDate: " + endDate);
            Console.WriteLine("Returned " + firstSearch.Length() + " documents.");
            Extractor extractor = new Extractor(firstSearch);
            SortedList<double, string> feat = extractor.GetIndex(Globals.PhraseRegex).GetSortedIndex();
            using (StreamWriter wr = new StreamWriter("c:\\" + queryString[0] + "features.txt",false, System.Text.Encoding.Default))
            {
                foreach (double key in feat.Keys)
                {
                    wr.WriteLine(key+" "+feat[key]);
                }
            }
            //StoreHitsAsFile(firstSearch, "c:\\statoil_case1.txt");

            ExtractorIndex companyIndex = extractor.GetIndex(Globals.CompanyRegex);

            ExtractorIndex personIndex = extractor.GetIndex(Globals.PersonRegex);

            SortedList<string, SortedList<double, string>> features = new SortedList<string, SortedList<double, string>>();
            
            SortedList<string, string> similarityList = new SortedList<string, string>();
            foreach (string key in companyIndex.GetKeys())
            {
                Console.WriteLine("Company: " + key);
                string[] company = key.Split(' ');
                Hits companyHits = searcher.SearchForConcepts(company, startDate, endDate);
                Extractor e2 = new Extractor(companyHits);
                string similarity = extractor.GetIndex(Globals.PhraseRegex).Compare(e2.GetIndex(Globals.PhraseRegex)).ToString();
                Hits t = searcher.SearchForConcepts(queryString, company, startDate, endDate);
                Extractor e3 = new Extractor(t);
                features[key] = e3.GetIndex(Globals.PhraseRegex).GetSortedIndex();
                //stw.WriteLine(key + "\tsimilarity: " + similarity + "\t#hits" + t.Length());
                //TODO: Husk at dette fr konsekvenser ved to like verdier (eks 0).
                if (similarity.Equals("0"))
                {
                    similarity = "0,";
                }
                while (similarityList.ContainsKey(similarity))
                {
                    similarity = string.Concat(similarity,"0");
                }
                similarityList[similarity.ToString()] = key + "\t#hits: " + t.Length() + "\t";
            }

            using (StreamWriter stw = new StreamWriter("c:\\companies" + startDate.DayOfYear + endDate.DayOfYear + ".txt", false, System.Text.Encoding.Default))
            {
                stw.WriteLine("TFIDF? " + Globals.doTFIDFWeighting);
                stw.WriteLine("RegexPattern: " + Globals.PhraseRegex);
                foreach (string key in similarityList.Keys)
                {
                    stw.WriteLine(similarityList[key] + key);
                    string company = similarityList[key].Split('\t')[0].Trim();
                    
                    for (int i = 0;  i < features[company].Count; i++)
                    {
                        stw.WriteLine(features[company].Keys[i] + " " + features[company].Values[i]);
                    }
                }

            }
            //SortedList<string, SortedList<double, string>> features = new SortedList<string, SortedList<double, string>>();
            similarityList.Clear();
            foreach (string key in personIndex.GetKeys())
            {
                Console.WriteLine("Person: " + key);
                string[] person = key.Split(' ');
                if (person.Length > 1)
                {
                    Hits personHits = searcher.SearchForConcepts(person, startDate, endDate);
                    Extractor e2 = new Extractor(personHits);
                    string similarity = extractor.GetIndex(Globals.PhraseRegex).Compare(e2.GetIndex(Globals.PhraseRegex)).ToString();

                    Hits t = searcher.SearchForConcepts(queryString, person, startDate, endDate);
                    Extractor e3 = new Extractor(t);
                    features[key] = e3.GetIndex(Globals.PhraseRegex).GetSortedIndex();
                    if (similarity.Equals("0"))
                    {
                        similarity = "0,";
                    }
                    while (similarityList.ContainsKey(similarity))
                    {
                        similarity = string.Concat(similarity, "0");
                    }
                    similarityList[similarity] = key + "\t#hits: " + t.Length() + "\t";
                    //stw.WriteLine(key + "\tsimilarity: " + similarity + "\t#hits" + t.Length());
                }
            }
            using (StreamWriter stw = new StreamWriter("c:\\persons" + startDate.DayOfYear + endDate.DayOfYear + ".txt", false, System.Text.Encoding.Default))
            {
                stw.WriteLine("TFIDF? " + Globals.doTFIDFWeighting);
                stw.WriteLine("RegexPattern: " + Globals.PhraseRegex);
                    
                foreach (string key in similarityList.Keys)
                {
                    stw.WriteLine(similarityList[key] + key);
                    string person = similarityList[key].Split('\t')[0].Trim();
 /*
                    for (int i = 0; i<features[person].Count; i++)
                    {
                        stw.WriteLine(features[person].Keys[i] + " " + features[person].Values[i]);
                    }
   */                 
                }
            }
        }
    }
}