using System;
using System.Collections.Generic;
using System.Text;
using Lucene.Net.Search;
using Lucene.Net.QueryParsers;
using Lucene.Net.Index;
using Lucene.Net.Documents;
using Lucene.Net.Analysis;
using System.IO;

namespace ClassificationApplication.Search
{
    internal class ConceptSearcher
    {
        public string IndexDirectory
        {
            get { return Properties.Settings.Default.IndexDirectory; }
        }
        public Hits SearchForConcepts(string[] queryString, DateTime startDate, DateTime endDate)
        {
            Searcher searcher = null;

            try
            {
                searcher = new IndexSearcher(IndexDirectory);
            }
            catch (System.IO.IOException ioe)
            {
                Console.WriteLine("Unable to open index file. " + ioe);
            }

            BooleanQuery query = null;
            
            try
            {
                RangeQuery dateQuery = new RangeQuery(new Term("date", DateTools.DateToString(startDate, DateTools.Resolution.DAY)), new Term("date", DateTools.DateToString(endDate, DateTools.Resolution.DAY)), true);
                query = new BooleanQuery();
                query.Add(dateQuery, BooleanClause.Occur.MUST);
                BooleanQuery phraseFieldQuery = new BooleanQuery();
                foreach (string searchField in ClassificationApplication.FeatureExtractor.Globals.SearchFields)
                {
                    PhraseQuery phraseQuery = new PhraseQuery();
                    phraseQuery.SetSlop(2);
                    foreach (string partialPhrase in queryString)
                    {
                        phraseQuery.Add(new Term(searchField,partialPhrase));
                    }
                    //Det er nok at en av fields'ene inneholder frasen: BooleanClause.Occur.SHOULD
                    phraseFieldQuery.Add(phraseQuery, BooleanClause.Occur.SHOULD);
                }
                query.Add(phraseFieldQuery, BooleanClause.Occur.MUST);
            }

            catch (ParseException e)
            {
                searcher.Close();
                Console.WriteLine("Unable to parse: " + queryString + "\n" + e);
            }
            Hits hits = null;
            try
            {
                hits = searcher.Search(query);
                List<DateTime> dates = new List<DateTime>();
                for (int i = 0; i < hits.Length(); i++)
                {
                    string dateField = hits.Doc(i).GetField("date").StringValue();
                    DateTime date = new DateTime(int.Parse(dateField.Substring(0, 4)), int.Parse(dateField.Substring(4, 2)), int.Parse(dateField.Substring(6, 2)));
                    dates.Add(date);
                }
                dates.Sort();
                DateTime currentDate = new DateTime(2006,1,2);
                List<int> articlesInWeek = new List<int>();
                List<DateTime> currentWeek = new List<DateTime>();
                while (currentDate < DateTime.Now)
                {
                    foreach (DateTime day in dates)
                    {
                        if (day.CompareTo(currentDate) >= 0 && day.CompareTo(currentDate.AddDays(7)) < 0)
                        {
                            currentWeek.Add(day);
                        }
                    }
                    articlesInWeek.Add(currentWeek.Count);
                    currentDate = currentDate.AddDays(7);
                    currentWeek = new List<DateTime>();
                }/*
                using (StreamWriter stw = new StreamWriter("c:\\" + queryString[0], false, Encoding.Default))
                {
                    int i = 1;
                    foreach (int articlesThisWeek in articlesInWeek)
                    {
                        stw.WriteLine("Uke: " + i + "     " + articlesThisWeek);
                        i++;
                    } 
                }*/
            }
            catch (System.IO.IOException e)
            {
                searcher.Close();
                Console.WriteLine("IO Error.\n" + e);
                Console.WriteLine(e.StackTrace);
            }
            return hits;
        }



        public Hits SearchForConcepts(string[] partialPhrase, string[] partialPhrase2, DateTime startDate, DateTime endDate)
        {
            Searcher searcher = null;

            try
            {
                searcher = new IndexSearcher(IndexDirectory);
            }
            catch (System.IO.IOException ioe)
            {
                Console.WriteLine("Unable to open index file. " + ioe);
            }

            BooleanQuery query = null;

            try
            {
                RangeQuery dateQuery = new RangeQuery(new Term("date", DateTools.DateToString(startDate, DateTools.Resolution.DAY)), new Term("date", DateTools.DateToString(endDate, DateTools.Resolution.DAY)), true);
                query = new BooleanQuery();
                query.Add(dateQuery, BooleanClause.Occur.MUST);
                BooleanQuery phraseFieldQuery = new BooleanQuery();
                BooleanQuery phraseFieldQuery2 = new BooleanQuery();
                foreach (string searchField in ClassificationApplication.FeatureExtractor.Globals.SearchFields)
                {
                    PhraseQuery phraseQuery = new PhraseQuery();
                    phraseQuery.SetSlop(2);
                    foreach (string pP in partialPhrase)
                    {
                        phraseQuery.Add(new Term(searchField, pP));
                    }
                    //Det er nok at en av fields'ene inneholder frasen: BooleanClause.Occur.SHOULD
                    phraseFieldQuery.Add(phraseQuery, BooleanClause.Occur.SHOULD);
                    PhraseQuery phraseQuery2 = new PhraseQuery();
                    phraseQuery2.SetSlop(2);
                    foreach (string pP in partialPhrase2)
                    {
                        phraseQuery2.Add(new Term(searchField, pP));
                    }
                    phraseFieldQuery2.Add(phraseQuery2, BooleanClause.Occur.SHOULD);
                }
                query.Add(phraseFieldQuery, BooleanClause.Occur.MUST);
                query.Add(phraseFieldQuery2, BooleanClause.Occur.MUST);
            }

            catch (ParseException e)
            {
                searcher.Close();
                Console.WriteLine("Unable to parse: " + partialPhrase + "\n" + e);
            }
            Hits hits = null;
            try
            {
                hits = searcher.Search(query);
                List<DateTime> dates = new List<DateTime>();
                for (int i = 0; i < hits.Length(); i++)
                {
                    string dateField = hits.Doc(i).GetField("date").StringValue();
                    DateTime date = new DateTime(int.Parse(dateField.Substring(0, 4)), int.Parse(dateField.Substring(4, 2)), int.Parse(dateField.Substring(6, 2)));
                    dates.Add(date);
                }
                dates.Sort();
                DateTime currentDate = new DateTime(2006, 1, 2);
                List<int> articlesInWeek = new List<int>();
                List<DateTime> currentWeek = new List<DateTime>();
                while (currentDate < DateTime.Now)
                {
                    foreach (DateTime day in dates)
                    {
                        if (day.CompareTo(currentDate) >= 0 && day.CompareTo(currentDate.AddDays(7)) < 0)
                        {
                            currentWeek.Add(day);
                        }
                    }
                    articlesInWeek.Add(currentWeek.Count);
                    currentDate = currentDate.AddDays(7);
                    currentWeek = new List<DateTime>();
                }
                /*
                using (StreamWriter stw = new StreamWriter("c:\\" + partialPhrase[0], false, Encoding.Default))
                {
                    int i = 1;
                    foreach (int articlesThisWeek in articlesInWeek)
                    {
                        stw.WriteLine("Uke: " + i + "     " + articlesThisWeek);
                        i++;
                    }
                }
                 */ 
            }
            catch (System.IO.IOException e)
            {
                searcher.Close();
                Console.WriteLine("IO Error.\n" + e);
                Console.WriteLine(e.StackTrace);
            }
            return hits;
        }
    }
}
