using System;
using System.Collections;
using System.Text;
using System.Net;
using System.IO;
using log4net;

//[assembly: log4net.Config.XmlConfigurator(Watch = true)]
namespace NewsFetcher.ArticleFetcher
{

    class ArticleFetcher
    {
        private ArticleStore store;

        private static readonly log4net.ILog log = log4net.LogManager.GetLogger(System.Reflection.MethodBase.GetCurrentMethod().DeclaringType);
        
        public ArticleFetcher()
        {
            log.Info("Intializing ArticleFetcher");
            store = new ArticleStore();
        }
        
        public void FetchNewsArticle(string url)
        {
            Article article;
            if (store.IsArticleStored(url))
            {
                log.Info("Article has already been stored: " + url);
            }
            else
            {
                try
                {
                    if (url.Contains("http://www.dn.no"))
                    {
                        article = FetchDNArticle(url);
                    }
                    else if (url.Contains("http://www.hegnar.no"))
                    {
                        article = FetchHegnarArticle(url);
                    }
                    else if (url.Contains("http://www.aftenposten.no/nyheter/okonomi"))
                    {
                        article = FetchAftenpostenArticle(url);
                    }
                    else if (url.Contains("http://www.imarkedet.no"))
                    {
                        article = FetchiMarkedetArticle(url);
                    }
                    else if (url.Contains("http://pub.tv2.no/nettavisen"))
                    {
                        article = FetchTV2Article(url);
                    }
                    else if (url.Contains("http://www.siste.no"))
                    {
                        article = FetchSisteArticle(url);
                    }
                    else
                    {
                        log.Warn("Could not fetch the url: " + url);
                        return;
                    }
                }
                catch (ApplicationException ae)
                {
                    log.Warn("URL cannot be empty. " + url + "\n" + ae);
                    return;
                }
                catch (Exception e)
                {
                    log.Warn("Exception caught while fetching the url: " + url + "\n" + e);
                    return;
                }
                log.Info("Fetched a new article:" + article);
                store.StoreArticle(article);
            }
        }

        private Article FetchDNArticle(string url)
        {
            string html = this.GetHTMLStream(url);
            string title;
            string h1;
            string ingress;
            string paragraph;
            ArrayList paragraphList = new ArrayList();
            string tmpHtml = html;

            /*Extract title*/
            tmpHtml = tmpHtml.Substring(tmpHtml.IndexOf("<title>") + 7);//+7 for  fjerne <title> tag
            title = tmpHtml.Substring(0,tmpHtml.IndexOf("</title>"));
            title = RemoveDnFormatting(title);

            /*Extract h1*/
            tmpHtml = tmpHtml.Substring(tmpHtml.IndexOf("<h1>") + 4);
            h1 = tmpHtml.Substring(0, tmpHtml.IndexOf("</h1>"));
            h1 = RemoveDnFormatting(h1);

            /*Extract ingress*/
            tmpHtml = tmpHtml.Substring(tmpHtml.IndexOf("<div class=\"leadText\">") + 22);
            ingress = tmpHtml.Substring(0, tmpHtml.IndexOf("</div>"));
            ingress = ingress.Replace("<p>", "");
            ingress = ingress.Replace("</p>","");
            ingress = RemoveDnFormatting(ingress);

            /*Extract paragraphs*/
            tmpHtml = tmpHtml.Substring(tmpHtml.IndexOf("<div class=\"body\">") + 18);
            tmpHtml = tmpHtml.Substring(0, tmpHtml.IndexOf("</div>"));
                /*First paragraph*/
            tmpHtml = tmpHtml.Substring(tmpHtml.IndexOf("<p>") + 3);
            paragraph = tmpHtml.Substring(0, tmpHtml.IndexOf("</p>"));
            paragraph = RemoveDnFormatting(paragraph);
            paragraphList.Add(paragraph);

                /*The remaining of the paragraphs*/
            while (tmpHtml.Contains("<p class=\"next\">"))
            {
                tmpHtml = tmpHtml.Substring(tmpHtml.IndexOf("<p class=\"next\">") + 16);
                paragraph = tmpHtml.Substring(0, tmpHtml.IndexOf("</p>"));
                paragraph = RemoveDnFormatting(paragraph);
                paragraphList.Add(paragraph);
                //System.Console.WriteLine("Avsnitt" + paragraph);
            }

            string[] paragraphArray = (String [])paragraphList.ToArray(typeof(String) );
            
            Article article = new Article(url, h1, ingress, paragraphArray);

            return article;
        }
        private string RemoveDnFormatting(string text)
        {
            if (text.Contains("Les ogs"))
            {
                //Console.WriteLine("Fjernet avsnitt fordi det inneholder \" Les ogs\": " + paragraph);
                return null;
            }
            while (text.Contains("<LI>"))
            {
                text = text.Replace("<LI>", ". ");
            }
            while (text.Contains("<li>"))
            {
                text = text.Replace("<li>", ". ");
            }
            while (text.Contains("<br>"))
            {
                text = text.Replace("<br>", ". ");
            }
            while (text.Contains("<BR>"))
            {
                text = text.Replace("<BR>", ". ");
            }
            while (text.Contains("&#8211;"))
            {
                text = text.Replace("&#8211;", "-");
            }
            while (text.Contains("&amp;"))
            {
                text = text.Replace("&amp;", "&");
            }
            while (text.Contains("&laquo;"))
            {
                text = text.Replace("&laquo;", "");
            }
            while (text.Contains("&raquo;"))
            {
                text = text.Replace("&raquo;", "");
            }
            while (text.Contains("&nbsp;"))
            {
                text = text.Replace("&nbsp;", " ").Trim();
            }
            while (text.Contains("(NTB)"))
            {
                text = text.Replace("(NTB)", " ").Trim();
            }
            while (text.Contains("&#8217;"))
            {
                text = text.Replace("&#8217;", " ").Trim();
            }
            while (text.Contains("&#8221;"))
            {
                text = text.Replace("&#8221;", " ").Trim();
            }
            while (text.Contains("&ndash;"))
            {
                text = text.Replace("&ndash;", " ").Trim();
            }
            text = RemoveTags(text);
            return text;
        }
        public static string RemoveTags(string text)
        {
            while (text.Contains("<"))
            {
                string tmpText = text.Substring(0, text.IndexOf("<")).Trim();
                if (tmpText.Length > 0)
                {
                    tmpText = String.Concat(tmpText, " ");
                }
                //Console.WriteLine("Fr fjerning: " + paragraph);
                if (text.Contains(">"))
                {
                    text = tmpText + text.Substring(text.IndexOf(">") + 1);
                    //Console.WriteLine("Etter fjerning: " + paragraph);
                }
                else
                {
                    break;
                }
            }
            while (text.Contains("'"))
            {
                text = text.Replace("'", "");
            }
            return text;
        }
        private Article FetchHegnarArticle(string url)
        {
            string html = this.GetHTMLStream(url);
            string title;
            string ingress;
            string paragraph;
            ArrayList paragraphList = new ArrayList();
            string _tmpHtml = html;

            /*Remove everything after the article content*/
            _tmpHtml = _tmpHtml.Substring(0, _tmpHtml.IndexOf("<!-- END ARTICKLE SECTION -->"));

            /*Extract title*/
            _tmpHtml = _tmpHtml.Substring(_tmpHtml.IndexOf("<td CLASS=\"fa14\" name=\"Title\">") + 30);
            title = _tmpHtml.Substring(0, _tmpHtml.IndexOf("</td>"));

            /*Extract ingress*/
            _tmpHtml = _tmpHtml.Substring(_tmpHtml.IndexOf("<td CLASS=\"fa11\" name=\"Ingress\">") + 32);
            ingress = _tmpHtml.Substring(0, _tmpHtml.IndexOf("</td>"));

            //Console.WriteLine("Title: "+title);
            //Console.WriteLine("Ingress: "+ingress);

            /*Extract paragraphs*/
            string unFormattedParagraph;
            _tmpHtml = _tmpHtml.Substring(_tmpHtml.IndexOf("<td CLASS=\"fa10\" name=\"artBody\">") + 32);
            string tmpHtml = _tmpHtml;
            
            if (_tmpHtml.Contains("<P") && !_tmpHtml.Contains("DIV"))
            {//Smstygg hack for  fikse problem nedenfor.. 
                unFormattedParagraph = _tmpHtml.Substring(0, _tmpHtml.IndexOf("<P"));
                if (unFormattedParagraph.Length > 0)
                {
                    unFormattedParagraph = RemoveDnFormatting(unFormattedParagraph);
                    if (unFormattedParagraph.Trim().Length > 0)
                    {
                        paragraphList.Add(unFormattedParagraph);
                    }
                }
            }
            while (_tmpHtml.Contains("<P"))
            {//TODO: Test denne. Her fjernes alt som ikke ligger mellom <p> og </p>.
                _tmpHtml = _tmpHtml.Substring(_tmpHtml.IndexOf("<P") + 2);
                _tmpHtml = _tmpHtml.Substring(_tmpHtml.IndexOf(">") + 1);
                int stringEnd = int.MaxValue;
                if (_tmpHtml.IndexOf("</P>") > 0)
                {
                    stringEnd = _tmpHtml.IndexOf("</P>");
                }
                if (_tmpHtml.IndexOf("<P>") > 0 && _tmpHtml.IndexOf("<P>") < stringEnd)
                {
                    stringEnd = _tmpHtml.IndexOf("<P>");
                }
                /*if (stringEnd > 0 && stringEnd < int.MaxValue)
                {

                }*/
                paragraph = _tmpHtml.Substring(0, stringEnd/*html.IndexOf("</P>")*/);
                paragraph = RemoveDnFormatting(paragraph);
                if (paragraph.Trim().Equals("&nbsp;"))
                {
                    Console.WriteLine("Rart avsnitt: " + paragraph);
                    continue;
                }
                if (paragraph.Trim().Length > 0)
                {
                    paragraphList.Add(paragraph);
                    //Console.WriteLine(unFormattedParagraph);
                    //Console.WriteLine(paragraph);
                }
            }
            if (paragraphList.Count < 2)
            {
                string prevParagraph = "";
                _tmpHtml = tmpHtml;
                while (_tmpHtml.Contains("<DIV"))
                {//TODO: Test denne. Her fjernes alt som ikke ligger mellom <p> og </p>.

                    _tmpHtml = _tmpHtml.Substring(_tmpHtml.IndexOf("<DIV") + 4);
                    _tmpHtml = _tmpHtml.Substring(_tmpHtml.IndexOf(">") + 1);
                    paragraph = _tmpHtml.Substring(0, _tmpHtml.IndexOf("</DIV>"));
                    paragraph = RemoveDnFormatting(paragraph);
                    if (paragraph.Trim().Equals("&nbsp;"))
                    {
                        continue;
                    }
                    if (!prevParagraph.Equals(paragraph))
                    {//Dirty bugfix (same paragraph appears twice sometimes...)
                        if (paragraph.Trim().Length > 0)
                        {
                            paragraphList.Add(paragraph);
                            prevParagraph = paragraph;
                            //Console.WriteLine(paragraph);
                        }
                    }
                }
                /*
                if (paragraphList.Count < 2)
                {
                    paragraph = RemoveTags(_tmpHtml);
                    if (paragraph.Trim().Length > 0)
                    {
                        paragraphList.Add(paragraph);
                    }
                }*/
            }
            string[] paragraphArray = (String[])paragraphList.ToArray(typeof(String));
            Article article = new Article(url, title, ingress, paragraphArray);
            return article;
        }
        private Article FetchAftenpostenArticle(string url)
        {
            string html = this.GetHTMLStream(url);
            string title;
            //string h1;
            string ingress;
            string paragraph;
            ArrayList paragraphList = new ArrayList();
            string tmpHtml = html;

            /*Remove everything after the article content*/
            tmpHtml = tmpHtml.Substring(0, tmpHtml.IndexOf("Tips en venn"));

            /*Extract title*/
            tmpHtml = tmpHtml.Substring(tmpHtml.IndexOf("Publisert:") + 10);
            if (tmpHtml.Contains("Oppdatert:"))
            {
                tmpHtml = tmpHtml.Substring(tmpHtml.IndexOf("Oppdatert:") + 10);
            }
            tmpHtml = tmpHtml.Substring(tmpHtml.IndexOf(":") + 3);
            tmpHtml = DecodeNorwegianLetters(tmpHtml);
            title = tmpHtml.Substring(0, tmpHtml.IndexOf("</h1>"));
            title = RemoveTags(title);

            //Console.WriteLine("Tittel: " + title);
       
            /*Extract ingress*/
            ingress = tmpHtml.Substring(tmpHtml.IndexOf("<span class=\"artIngress\">") + 25);
            ingress = ingress.Substring(0, ingress.IndexOf("</span>"));
            ingress = RemoveTags(ingress);

            //Console.WriteLine("Ingress: "+ingress); 

            /*Extract paragraphs*/
            tmpHtml = tmpHtml.Substring(tmpHtml.IndexOf("</span>") + 7);
            tmpHtml = tmpHtml.Substring(tmpHtml.IndexOf("</table>") + 8);
            
            while(tmpHtml.Contains("<p"))
            {
                paragraph = tmpHtml;
                //Console.WriteLine("HTML fr: "+html);
                if (tmpHtml.IndexOf("<p") > 0)
                {
                    tmpHtml = tmpHtml.Substring(tmpHtml.IndexOf("<p"));
                    paragraph = paragraph.Substring(0, paragraph.IndexOf("<p"));
                }
                else if (tmpHtml.IndexOf("<p") == 0)
                {
                    paragraph = paragraph.Substring(0, paragraph.IndexOf("</p"));
                    tmpHtml = tmpHtml.Substring(tmpHtml.IndexOf("</p"));
                }
                //Console.WriteLine("HTML etter: " + html);
                paragraph = RemoveTags(paragraph);
                paragraph = DecodeNorwegianLetters(paragraph);
                
                while (paragraph.Contains("(&copy;NTB)"))
                {
                    paragraph = paragraph.Replace("(&copy;NTB)", "");
                }
                while (paragraph.Contains("(&copy; NTB)"))
                {
                    paragraph = paragraph.Replace("(&copy; NTB)", "");
                }
                while(paragraph.Contains("(Aftenposten.no)"))
                {
                    paragraph = paragraph.Replace("(Aftenposten.no)", "");
                }
                while (paragraph.Contains("&#160;"))
                {
                    paragraph = paragraph.Replace("&#160;", "");
                }

                if (paragraph.Trim().Length > 1)
                {
                    //Console.WriteLine("Avsnitt: " + paragraph);
                    paragraphList.Add(paragraph);
                }
            }
            paragraph = tmpHtml;
            while (paragraph.Contains("(&copy;NTB)"))
            {
                paragraph = paragraph.Replace("(&copy;NTB)", "");
            }
            while (paragraph.Contains("(&copy; NTB)"))
            {
                paragraph = paragraph.Replace("(&copy; NTB)", "");
            }
            while (paragraph.Contains("(Aftenposten.no)"))
            {
                paragraph = paragraph.Replace("(Aftenposten.no)", "");
            }
            while (paragraph.Contains("&#160;"))
            {
                paragraph = paragraph.Replace("&#160;", "");
            }
            paragraph = RemoveTags(paragraph);
            paragraph = DecodeNorwegianLetters(paragraph);
            if (paragraph.Trim().Length > 1)
            {
                paragraphList.Add(paragraph);
            }
            
            string[] paragraphArray = (String[])paragraphList.ToArray(typeof(String));
            Article article = new Article(url, title, ingress, paragraphArray);

            return article;
        }
        private string DecodeNorwegianLetters(string text)
        {
            while (text.Contains("&oslash;"))
            {
                text = text.Replace("&oslash;", "");
            }
            while (text.Contains("&Oslash;"))
            {
                text = text.Replace("&Oslash;", "");
            }
            while (text.Contains("&aring;"))
            {
                text = text.Replace("&aring;", "");
            }
            while (text.Contains("&Aring;"))
            {
                text = text.Replace("&Aring;", "");
            }
            while (text.Contains("&aelig;"))
            {
                text = text.Replace("&aelig;","");
            }
            while (text.Contains("&Aelig;"))
            {
                text = text.Replace("&Aelig;", "");
            }
            while (text.Contains("&#214;"))
            {
                text = text.Replace("&#214;", "");
            }
            while (text.Contains("&#146;"))
            {
                text = text.Replace("&#146;", "'");
            }
            while (text.Contains("&#150;"))
            {
                text = text.Replace("&#150;", "-");
            }
            while (text.Contains("&#233;"))
            {
                text = text.Replace("&#233;", "e");
            }
            while (text.Contains("&laquo;"))
            {
                text = text.Replace("&laquo;", "\"");
            }
            while (text.Contains("&raquo;"))
            {
                text = text.Replace("&raquo;", "\"");
            }
            while (text.Contains("&ndash;"))
            {
                text = text.Replace("&ndash;", "-");
            }
            while (text.Contains("&ouml;"))
            {
                text = text.Replace("&ouml;", "");
            }
            while (text.Contains("&Ouml;"))
            {
                text = text.Replace("&Ouml;", "");
            }
            while (text.Contains("&eacute;"))
            {
                text = text.Replace("&eacute;", "");
            }
            while (text.Contains("&Eacute;"))
            {
                text = text.Replace("&Eacute;", "");
            }
            while (text.Contains("&#252;"))
            {
                text = text.Replace("&#252;", "");
            }
            while (text.Contains("&#171;"))
            {
                text = text.Replace("&#171;", "\"");
            }
            while (text.Contains("&#187;"))
            {
                text = text.Replace("&#187;", "\"");
            }
            while (text.Contains("&rsquo;"))
            {
                text = text.Replace("&rsquo;", "'");
            }
            while (text.Contains("(&#169;NTB)"))
            {
                text = text.Replace("(&#169;NTB)", "");
            }
            while (text.Contains("(NA24):"))
            {
                text = text.Replace("(NA24):", "");
            }
            while (text.Contains("(NA24)"))
            {
                text = text.Replace("(NA24)", "");
            }
            return text;
        }
        private Article FetchiMarkedetArticle(string url)
        {
            string html = this.GetHTMLStream(url);
            string title;
            //string h1;
            string ingress;
            string paragraph;
            ArrayList paragraphList = new ArrayList();
            string tmpHtml = html;

            /*Remove everything after the article content*/
            tmpHtml = tmpHtml.Substring(0, tmpHtml.IndexOf("<br /><br />"));
            tmpHtml = DecodeNorwegianLetters(tmpHtml);

            /*Extract title*/
            tmpHtml = tmpHtml.Substring(tmpHtml.IndexOf("<div class=\"story-headline\">") + 28);
            title = tmpHtml.Substring(0, tmpHtml.IndexOf("</div>"));
            title = RemoveTags(title);

            /*Extract ingress*/
            ingress = tmpHtml.Substring(tmpHtml.IndexOf("<div id=\"story-ingress\" class=\"story-ingress\">") + 46);
            ingress = ingress.Substring(0, ingress.IndexOf("</div>"));
            ingress = RemoveTags(ingress);
            ingress = DecodeNorwegianLetters(ingress);
 

            /*Extract paragraphs*/
            tmpHtml = tmpHtml.Substring(tmpHtml.IndexOf("<div id=\"ds_bodytext\" class=\"bodytext\" style=\"margin:10 0 0 3\" >") + 64);
            while (tmpHtml.Contains("<br>"))
            {
                paragraph = tmpHtml.Substring(0, tmpHtml.IndexOf("<br>"));
                paragraph = RemoveTags(paragraph);
                if (paragraph.Length > 1)
                {
                    paragraphList.Add(paragraph);
                }
                tmpHtml = tmpHtml.Substring(tmpHtml.IndexOf("<br>") + 4);
            }

            paragraph = RemoveTags(tmpHtml);
            if (paragraph.Length > 1)
            {
                paragraphList.Add(paragraph);
            }
            string[] paragraphArray = (String[])paragraphList.ToArray(typeof(String));
            Article article = new Article(url, title, ingress, paragraphArray);

            return article;
        }
        private Article FetchTV2Article(string url) 
        {
            string html = this.GetHTMLStream(url);
            string title;
            string ingress;
            string paragraph = "";
            ArrayList paragraphList = new ArrayList();

            /*Remove everything after the article content*/
            html = html.Substring(0, html.IndexOf("<!--Dato-->"));
            html = DecodeNorwegianLetters(html);

            /*Extract title*/
            html = html.Substring(html.IndexOf("<h1 class=\"title\">") + 18);
            title = html.Substring(0, html.IndexOf("</h1>"));
            title = RemoveTags(title);
            title = DecodeNorwegianLetters(title);

            /*Extract ingress*/
            ingress = html.Substring(html.IndexOf("<div class=\"leadtext\">") + 22);
            ingress = ingress.Substring(0, ingress.IndexOf("</div>"));
            ingress = RemoveTags(ingress);
            ingress = DecodeNorwegianLetters(ingress);

            html = html.Substring(html.IndexOf("<!--Brdtekst-->"));

            while (html.Contains("<p"))
            {
                paragraph = html;
                if (html.IndexOf("<p") > 0)
                {
                    html = html.Substring(html.IndexOf("<p"));
                    paragraph = paragraph.Substring(0, paragraph.IndexOf("<p"));
                }
                else if (html.IndexOf("<p") == 0)
                {
                    paragraph = paragraph.Substring(0, paragraph.IndexOf("</p"));
                    html = html.Substring(html.IndexOf("</p"));
                }
                paragraph = RemoveTags(paragraph);
                paragraph = DecodeNorwegianLetters(paragraph);

                while (paragraph.Contains("&bull;"))
                {
                    paragraph = paragraph.Replace("&bull;", "");
                }
                if (paragraph.Trim().Length > 1)
                {
                    paragraphList.Add(paragraph);
                }
            }
            paragraph = html;
            paragraph = RemoveTags(paragraph);
            paragraph = DecodeNorwegianLetters(paragraph);
            while (paragraph.Contains("&bull;"))
            {
                paragraph = paragraph.Replace("&bull;", "");
            }
            if (paragraph.Trim().Length > 1)
            {
                paragraphList.Add(paragraph);
            }

            string[] paragraphArray = (String[])paragraphList.ToArray(typeof(String));
            Article article = new Article(url, title, ingress, paragraphArray);

            return article;
        }
        private Article FetchSisteArticle(string url) 
        {
            string html = this.GetHTMLStream(url);
            string title;
            //string h1;
            string ingress;
            string paragraph;
            ArrayList paragraphList = new ArrayList();
            string tmpHtml = html;

            /*Remove everything after the article content*/
            tmpHtml = tmpHtml.Substring(0, tmpHtml.IndexOf("<div class=\"apiDate\">"));
            tmpHtml = DecodeNorwegianLetters(tmpHtml);

            /*Extract title*/
            tmpHtml = tmpHtml.Substring(tmpHtml.IndexOf("<span class=\"apiTitle\">") + 23);
            title = tmpHtml.Substring(0, tmpHtml.IndexOf("</span>"));
            title = RemoveTags(title);
            title = RemoveSisteFormatting(title);

            /*Extract ingress*/
            ingress = tmpHtml.Substring(tmpHtml.IndexOf("<p class=\"apiLeadText\">") + 23);
            ingress = ingress.Substring(0, ingress.IndexOf("</p>"));
            ingress = RemoveTags(ingress);
            ingress = DecodeNorwegianLetters(ingress);
            ingress = RemoveSisteFormatting(ingress);

            tmpHtml = tmpHtml.Substring(tmpHtml.IndexOf("<p class=\"apiBody\">") + 19);
            tmpHtml = tmpHtml.Substring(0, tmpHtml.IndexOf("<div class=\"apiTools\">"));

            while (tmpHtml.Contains("<p"))
            {
                paragraph = tmpHtml;
                //Console.WriteLine("HTML fr: "+html);
                if (tmpHtml.IndexOf("<p") > 0)
                {
                    tmpHtml = tmpHtml.Substring(tmpHtml.IndexOf("<p"));
                    paragraph = paragraph.Substring(0, paragraph.IndexOf("<p"));
                    //Console.WriteLine("1");
                }
                else if (tmpHtml.IndexOf("<p") == 0)
                {
                    paragraph = paragraph.Substring(0, paragraph.IndexOf("</p"));
                    tmpHtml = tmpHtml.Substring(tmpHtml.IndexOf("</p"));
                    //Console.WriteLine("2");
                }
                //Console.WriteLine("HTML etter: " + html);
                paragraph = RemoveTags(paragraph);
                paragraph = DecodeNorwegianLetters(paragraph);
                paragraph = RemoveSisteFormatting(paragraph);
                if (paragraph.Length > 1)
                {
                    //Console.WriteLine("Avsnitt: " + paragraph);
                    paragraphList.Add(paragraph);
                }
            }
            paragraph = tmpHtml;
            paragraph = RemoveTags(paragraph);
            paragraph = DecodeNorwegianLetters(paragraph);
            paragraph = RemoveSisteFormatting(paragraph);
            if (paragraph.Length > 1)
            {
                paragraphList.Add(paragraph);
            }


            string[] paragraphArray = (String[])paragraphList.ToArray(typeof(String));
            Article article = new Article(url, title, ingress, paragraphArray);

            return article;

        }
        private string RemoveSisteFormatting(string text)
        {
            while (text.Contains("(ANB)"))
            {
                text = text.Replace("(ANB)", "");
            }
            while (text.Contains("&laquo;"))
            {
                text = text.Replace("&laquo;", "\"");
            }
            while (text.Contains("&raquo;"))
            {
                text = text.Replace("&raquo;", "\"");
            }
            while(text.Contains("&ndash;"))
            {
                text = text.Replace("&ndash;", "");
            }
            return text;
        }
        private string GetHTMLStream(string url)
        {
            if (url == null || url.Length == 0)
            {
                throw new ApplicationException("Specify the URI of the resource to retrieve.");
            }

            WebClient client = new WebClient();
            Stream data = client.OpenRead(url);
            StreamReader reader = new StreamReader(data, System.Text.Encoding.GetEncoding("ISO-8859-1"));//Handles norwegian ,,
            string s = reader.ReadToEnd();
            data.Close();
            reader.Close();
            return s;
        }
        /*
        public static void Main(string[] args)
        {
            ArticleFetcher fetcher = new ArticleFetcher();
            
            
              PrintArticle(fetcher.FetchHegnarArticle("http://www.hegnar.no/hegnar/newsdet.asp?id=209771"));
            //Console.WriteLine(article);
            
        }
        */ 
        public void FetchOldArticles(DateTime fromDate, DateTime toDate)
        {
            ArrayList urlList = store.FetchUrlsFromDB(fromDate, toDate);
            //Console.WriteLine("Antall url'er: " + urlList.Count);
            log.Info("Antall url'er" + urlList.Count);
            int unstored = 0;
            int stored = 0;
            
            foreach (string url in urlList)
            {
                if (store.IsArticleStored(url))
                {
                    stored++;
                    log.Info("Article has already been stored: " + url);
                }
                else
                {
                    if (!url.Equals("http://www.aftenposten.no/nyheter/okonomi/article1222165.ece") && !url.Equals("http://www.aftenposten.no/nyheter/okonomi/article1224274.ece") && !url.Equals("http://www.aftenposten.no/nyheter/okonomi/article1224781.ece"))//Noe feil skjer med denne
                    {
                        unstored++;
                        log.Info("Fetching " + url);
                        FetchNewsArticle(url);
                    }
                }
            }
            log.Info("Number of unstored articles: " + unstored);
            log.Info("Number of stored articles: " + stored);
        }
    }
}
