using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Diagnostics;
using System.ServiceProcess;
using System.Text;
using NewsFetcher.MessageFetcher;
using NewsFetcher.ArticleFetcher;
using System.Threading;

[assembly: log4net.Config.XmlConfigurator(Watch = true)]
namespace NewsFetcher
{
    partial class FetcherService : ServiceBase
    {
        private static readonly log4net.ILog log = log4net.LogManager.GetLogger("Service");
        private System.Threading.Thread messageFetcherThread;
        private System.Threading.Thread rssFetcherThread;
        private StockMessageFetcher messageFetcher;
        private RSSFetcher rssFetcher;

        public FetcherService()
        {
            InitializeComponent();
        }

        protected override void OnStart(string[] args)
        {
            //base.OnStart(args);
            messageFetcher = new StockMessageFetcher();
            messageFetcherThread = new Thread(new ThreadStart(messageFetcher.FetchStockMessages));
            messageFetcherThread.Start();

            rssFetcher = new NewsFetcher.ArticleFetcher.RSSFetcher();
            rssFetcherThread = new Thread(new ThreadStart(rssFetcher.RunRssFetching));
            rssFetcherThread.Start();

            log.Info("StockMessageFetcher and ArticleFetcher started");
        }

        protected override void OnStop()
        {
            //base.OnStop();
            while (rssFetcher.Fetching)
            {
                Thread.Sleep(5000);
            }
            rssFetcher.Running = false;
            log.Info("Stopped ArticleFetcher");

            while (messageFetcher.Fetching)
            {
                System.Threading.Thread.Sleep(5000);
            }
            messageFetcher.Running = false;

            log.Info("Stopped StockMessageFetcher");
        }

        protected override void OnShutdown()
        {
            log.Info("Shutting down ArticleFetcher");

            while (rssFetcher.Fetching)
            {
                Thread.Sleep(5000);
            }
            rssFetcher.Running = false;

            log.Info("Shutting down StockMessageFetcher");
            while (messageFetcher.Fetching)
            {
                System.Threading.Thread.Sleep(5000);
            }
            messageFetcher.Running = false;   
        }
    }
}
