using System;
using System.Collections;
using System.Text;
using System.Net;
using System.IO;
using log4net;
using System.Threading;

//[assembly: log4net.Config.XmlConfigurator(Watch = true)]
namespace NewsFetcher.MessageFetcher
{
    public class StockMessageFetcher
    {
        private static readonly log4net.ILog log = log4net.LogManager.GetLogger("NewsFetcher.MessageFetcher.StockMessageFetcher");

        private StockMessageStore store;
        private bool fetching;
        private bool running;

        public bool Fetching
        {
            get { return fetching; }
            set { fetching = value; }
        }
        public bool Running
        {
            get { return running; }
            set { running = value; }//TODO: Br vi interrupte trden og?
        }

        public StockMessageFetcher()
        {
            log.Info(System.Reflection.MethodBase.GetCurrentMethod().DeclaringType + "Initiated");
            this.store = new StockMessageStore();
            running = true;
            fetching = false;
            //log.Info("End of constructor");
            //Thread t = new Thread(new ThreadStart(FetchStockMessages));
            //t.Start();
            //FetchStockMessages();
        }
        public void FetchStockMessages()
        {
            while (running)
            {
                fetching = true;
                FetchStockMessages("http://www.stocklink.no/bin/Server?x-service=showMessages&numDays=60");
                fetching = false;
                System.Threading.Thread.Sleep(1000 * 60 * 60 * 2);//2 hours
                log.Info("Running a new iteration");
            }
        }
        private void FetchStockMessages(string url)
        {
            string title;
            string date;
            string time;
            DateTime dateTime;
            string ticker = "";
            string messageUrl;
            string stockString;
            string html = GetHTMLStream(url);

            html = html.Substring(html.IndexOf("<!-- liste over meldinger -->") + 29);

            html = html.Substring(0, html.IndexOf("<!-- hyre side, valg av meldinger -->"));

            while(html.Contains("<tr"))
            {
                stockString = html.Substring(html.IndexOf("<tr"), html.IndexOf("</tr>"));
                stockString = stockString.Substring(stockString.IndexOf("<a href=\""));
                messageUrl = stockString.Substring(9);
                messageUrl = messageUrl.Substring(0, messageUrl.IndexOf("\">"));
                messageUrl = string.Concat("http://www.stocklink.no", messageUrl);

                stockString = stockString.Substring(stockString.IndexOf("\">") + 2);
                title = stockString.Substring(0, stockString.IndexOf("</a>"));
                
                stockString = stockString.Substring(stockString.IndexOf("<td><font face=\"verdana,geneva,arial,helvetica\" size=\"1\">") + 57);
                date = stockString.Substring(0, stockString.IndexOf("</font>"));
                
                stockString = stockString.Substring(stockString.IndexOf("<td><font face=\"verdana,geneva,arial,helvetica\" size=\"1\">") + 57);
                time = stockString.Substring(0, stockString.IndexOf("</font>"));

                dateTime = new DateTime(2000 + int.Parse(date.Split(".".ToCharArray())[2]), int.Parse(date.Split(".".ToCharArray())[1]), int.Parse(date.Split(".".ToCharArray())[0]), int.Parse(time.Split(":".ToCharArray())[0]), int.Parse(time.Split(":".ToCharArray())[1]), int.Parse(time.Split(":".ToCharArray())[2]));
                if (stockString.Contains("<a href="))
                {
                    stockString = stockString.Substring(stockString.IndexOf("<a href="));
                    stockString = stockString.Substring(stockString.IndexOf("\">") + 2);
                    ticker = stockString.Substring(0, stockString.IndexOf("</a>"));
                }
                html = html.Substring(html.IndexOf("</tr>") + 5);
                if (!store.IsMessageStored(messageUrl))
                {
                    StockMessage message = new StockMessage(messageUrl, title, dateTime, FetchMessage(messageUrl), ticker);
                    store.StoreMessageInDB(message);
                    log.Info("Stored message in DB: \n"+ message);
                }
            }
        }

        private string FetchMessage(string url)
        {
            string html = GetHTMLStream(url);
            html = html.Substring(html.IndexOf("<!-- venstre side - meldinger -->") + 33);
            html = html.Substring(0, html.IndexOf("<!-- hyre side, valg av meldinger -->"));
            html = html.Substring(html.IndexOf("<pre>") + 5);
            html = html.Substring(0, html.IndexOf("</pre>"));

            return html;
        }

        private string GetHTMLStream(string url)
        {
            if (url == null || url.Length == 0)
            {
                throw new ApplicationException("Specify the URI of the resource to retrieve.");
            }

            WebClient client = new WebClient();
            Stream data = client.OpenRead(url);
            StreamReader reader = new StreamReader(data, System.Text.Encoding.GetEncoding("ISO-8859-1"));//Handles norwegian ,,
            string s = reader.ReadToEnd();
            data.Close();
            reader.Close();
            return s;
        }
        /*
        static void Main()
        {
            Console.WriteLine("Test");
            StockMessageFetcher fetcher = new StockMessageFetcher();
            fetcher.FetchStockMessages("http://www.stocklink.no/bin/Server?x-service=showMessages&numDays=1");
        }
         */ 
    }
    
}
