using System;
using System.Collections.Generic;
using System.Text;
using System.Data.SqlClient;
using System.Data;
using Lucene.Net.Documents;
using Lucene.Net.Index;
using Lucene.Net.Analysis;
using System.IO;

namespace Tagger.PropernameStore.Fetcher
{
   public class DBStore
    {
        private static string propernamesDirectory = "c:\\index\\propernames\\";

        public static string PropernamesDirectory
        {
            get { return DBStore.propernamesDirectory; }
            set { DBStore.propernamesDirectory = value; }
        }

        private static string personsFile = "c:\\index\\propernames\\persons.txt";

        public static string PersonsFile
        {
            get { return DBStore.personsFile; }
            set { DBStore.personsFile = value; }
        }
        private static string companiesFile = "c:\\index\\propernames\\companies.txt";

        public static string CompaniesFile
        {
            get { return DBStore.companiesFile; }
            set { DBStore.companiesFile = value; }
        }


       public static void StorePropernamesAsFiles()
       {
           StorePersonsAsFile();
           StoreCompaniesAsFile();
       }
       private static void StorePersonsAsFile()
       {
           using (SqlConnection connection = new SqlConnection(Properties.Settings.Default.CompanyDatabaseConnectionString))
           {
               try
               {
                   if (!Directory.Exists(PropernamesDirectory))
                   {
                       Directory.CreateDirectory(PropernamesDirectory);
                   }
                   connection.Open();
                   SqlDataAdapter pAdapter = new SqlDataAdapter();
                   SqlCommand sqlCommand = connection.CreateCommand();
                   sqlCommand.CommandText = "Select name from Person";
                   pAdapter.SelectCommand = sqlCommand;

                   DataSet dataSet = new DataSet();

                   pAdapter.Fill(dataSet, "Person");

                   DataRow[] personRows = dataSet.Tables["Person"].Select();

                   List<string> personList = new List<string>();
                   foreach (DataRow row in personRows)
                   {
                       personList.Add(row["name"].ToString());
                   }
                   personList.Sort();
                   using (StreamWriter writer = new StreamWriter(PersonsFile,false,Encoding.Default))
                   {
                       foreach (string person in personList)
                       {
                           writer.WriteLine(person);
                       }
                   }
               }
               catch (InvalidOperationException ioe)
               {
                   Console.Error.WriteLine("Failed to perform operation: " + ioe.Message);
               }
               catch (System.Data.SqlClient.SqlException ex)
               {
                   Console.Error.WriteLine("Failed to connect to data source: " + ex.Message);
               }
               finally
               {
                   connection.Close();
               }
           }
       }
       
       private static void StoreCompaniesAsFile()
       {
           using (SqlConnection connection = new SqlConnection(Properties.Settings.Default.CompanyDatabaseConnectionString))
           {
               try
               {
                   if (!Directory.Exists(PropernamesDirectory))
                   {
                       Directory.CreateDirectory(PropernamesDirectory);
                   }
                   connection.Open();
                   SqlDataAdapter cAdapter = new SqlDataAdapter();
                   SqlCommand sqlCommand = connection.CreateCommand();
                   sqlCommand.CommandText = "Select name from Company";
                   cAdapter.SelectCommand = sqlCommand;

                   DataSet dataSet = new DataSet();

                   cAdapter.Fill(dataSet, "Company");

                   DataRow[] companyRows = dataSet.Tables["Company"].Select();

                   List<string> companiesList = new List<string>();
                   foreach (DataRow row in companyRows)
                   {
                       companiesList.Add(row["name"].ToString());
                   }
                   companiesList.Sort();
                   using (StreamWriter writer = new StreamWriter(CompaniesFile,false, Encoding.Default))
                   {
                       foreach (string company in companiesList)
                       {
                           writer.WriteLine(company);
                       }

                   }
               }
               catch (InvalidOperationException ioe)
               {
                   Console.Error.WriteLine("Failed to perform operation: " + ioe.Message);
               }
               catch (System.Data.SqlClient.SqlException ex)
               {
                   Console.Error.WriteLine("Failed to connect to data source: " + ex.Message);
               }
               finally
               {
                   connection.Close();
               }
           }
       }
        public bool IsPersonStored(string pId)
        {
            bool personStored = false;
            using (SqlConnection connection = new SqlConnection(Properties.Settings.Default.CompanyDatabaseConnectionString))
            {
                try
                {
                    connection.Open();
                    string selectPersonIdString = "select id from Person where id = '" + pId + "';";

                    SqlCommand selectPersonIdCommand = new SqlCommand(selectPersonIdString, connection);
                    SqlDataReader reader = selectPersonIdCommand.ExecuteReader();
                    personStored = true;
                }
                catch (InvalidOperationException ioe)
                {
                    Console.Error.WriteLine("Failed to perform operation: " + ioe.Message);
                }
                catch (System.Data.SqlClient.SqlException ex)
                {
                    Console.Error.WriteLine("Failed to connect to data source: " + ex.Message);
                }
                finally
                {
                    connection.Close();
                }
                return personStored;
            }
        }
        public bool IsCompanyStored(string orgNumber)
        {
            bool companyStored = false;
            using (SqlConnection connection = new SqlConnection(Properties.Settings.Default.CompanyDatabaseConnectionString))
            {
                try
                {
                    connection.Open();
                    string selectOrgNumberString = "select orgNumber from Company where orgNumber = '" + orgNumber + "';";

                    SqlCommand selectOrgNumberCommand = new SqlCommand(selectOrgNumberString, connection);
                    SqlDataReader reader = selectOrgNumberCommand.ExecuteReader();
                    companyStored = true;
                }
                catch (InvalidOperationException ioe)
                {
                    Console.Error.WriteLine("Failed to perform operation: " + ioe.Message);
                }
                catch (System.Data.SqlClient.SqlException ex)
                {
                    Console.Error.WriteLine("Failed to connect to data source: " + ex.Message);
                }
                finally
                {
                    connection.Close();
                }
                return companyStored;
            }
        }
        public void StoreCompany(Selskap selskap)
        {
            using (SqlConnection connection = new SqlConnection(Properties.Settings.Default.CompanyDatabaseConnectionString))
            {
                try
                {
                    connection.Open();
                    SqlDataReader reader;
                    if (!IsCompanyStored(selskap.Foretaksinformasjon.OrganisasjonsNr))
                    {
                        string keyWords = "";
                        if (selskap.FagomradeBransje.NokkelOrd != null)
                        {
                            foreach (string keyWord in selskap.FagomradeBransje.NokkelOrd)
                            {
                                keyWords = string.Concat(keyWord, ";");
                            }
                        }
                        string bransjer = "";
                        if (selskap.FagomradeBransje.Bransje != null)
                        {
                            foreach (string bransje in selskap.FagomradeBransje.Bransje)
                            {
                                bransjer = string.Concat(bransje, ";");
                            }
                        }
                        string cpvBransjer = "";
                        if (selskap.FagomradeBransje.CpvBransje != null)
                        {
                            foreach (string cpvBransje in selskap.FagomradeBransje.CpvBransje)
                            {
                                cpvBransjer = string.Concat(cpvBransje, ";");
                            }
                        }
                        string insertCompanyString = "insert into Company values('" + selskap.Foretaksinformasjon.OrganisasjonsNr + "','" + selskap.Foretaksinformasjon.JuridiskSelskapsnavn +
                            "','" + selskap.Foretaksinformasjon.Telefon + "','" + selskap.Foretaksinformasjon.Telefaks + "','" + selskap.Foretaksinformasjon.GateAdresse + "','" + selskap.Foretaksinformasjon.PostAdresse + "','" + selskap.Foretaksinformasjon.AksjeKapital + "','" + selskap.Foretaksinformasjon.AntallAnsatte + "','" + keyWords + "','" + bransjer + "','" + cpvBransjer + "');";
                        SqlCommand insertCompanyCommand = new SqlCommand(insertCompanyString, connection);
                        insertCompanyCommand.ExecuteNonQuery();

                        foreach (string naceBransje in selskap.FagomradeBransje.NaceBransje)
                        {
                            //TODO: Flytt denne splitten til Firma-structen
                            string bransjeKode = naceBransje.Substring(0, 6);
                            string bransjeTekst = naceBransje.Substring(6).Trim();
                            string selectBusinessectorString = "select businessectorNumber from NaceSector where businessectorNumber = '" + bransjeKode + "';";
                            SqlCommand selectOrgBusinessectorCommand = new SqlCommand(selectBusinessectorString, connection);
                            reader = selectOrgBusinessectorCommand.ExecuteReader();
                            if (!reader.HasRows)
                            {
                                reader.Close();
                                string insertBusinessectorString = "insert into NaceSector values('" + bransjeKode + "','" + bransjeTekst + "');";
                                SqlCommand insertBusinessectorCommand = new SqlCommand(insertBusinessectorString, connection);
                                insertBusinessectorCommand.ExecuteNonQuery();
                            }
                            reader.Close();

                            string selectCompanyInBusinessectorString = "select orgNumber from CompanyInNaceSector where orgNumber = '" + selskap.Foretaksinformasjon.OrganisasjonsNr + "' AND businessectorNumber = '" + bransjeKode + "';";
                            SqlCommand selectCompanyInBusinessectorCommand = new SqlCommand(selectCompanyInBusinessectorString, connection);
                            reader = selectCompanyInBusinessectorCommand.ExecuteReader();
                            if (!reader.HasRows)
                            {
                                reader.Close();
                                string insertCompanyInBusinessectorString = "insert into CompanyInNaceSector values('" + selskap.Foretaksinformasjon.OrganisasjonsNr + "','" + bransjeKode + "');";
                                SqlCommand insertCompanyInBusinessectorCommand = new SqlCommand(insertCompanyInBusinessectorString, connection);
                                insertCompanyInBusinessectorCommand.ExecuteNonQuery();
                            }
                            reader.Close();
                        }
                        if (selskap.Personer != null)
                        {
                            foreach (Person person in selskap.Personer)
                            {
                                string selectPersonString = "select id from Person where id = '" + person.Id + "';";
                                SqlCommand selectPersonCommand = new SqlCommand(selectPersonString, connection);
                                reader = selectPersonCommand.ExecuteReader();
                                if (!reader.HasRows)
                                {
                                    reader.Close();
                                    string insertPersonString = "insert into Person values('" + person.Id + "','" + person.Navn + "');";
                                    SqlCommand insertPersonCommand = new SqlCommand(insertPersonString, connection);
                                    insertPersonCommand.ExecuteNonQuery();
                                }
                                reader.Close();

                                string selectPersonInCompanyString = "select companyOrgNumber, personid, role from PersonInCompany where companyOrgNumber = '" + selskap.Foretaksinformasjon.OrganisasjonsNr + "' AND personid like '" + person.Id + "' AND role like '" + person.Rolle + "';";
                                SqlCommand selectPersonInCompanyCommand = new SqlCommand(selectPersonInCompanyString, connection);
                                reader = selectPersonInCompanyCommand.ExecuteReader();
                                if (!reader.HasRows)
                                {
                                    reader.Close();
                                    string insertPersonInCompanyString = "insert into PersonInCompany values('" + person.Id + "','" + selskap.Foretaksinformasjon.OrganisasjonsNr + "','" + person.Rolle + "');";
                                    SqlCommand insertPersonInCompanyCommand = new SqlCommand(insertPersonInCompanyString, connection);
                                    insertPersonInCompanyCommand.ExecuteNonQuery();
                                }
                                reader.Close();
                            }
                        }
                        foreach (DatterSelskap dc in selskap.Foretaksinformasjon.DatterSelskaper)
                        {
                            if (dc.Link.Length > 5)
                            {
                                string insertDaughterCompanyString = "insert into DaughterCompany values('" + selskap.Foretaksinformasjon.OrganisasjonsNr + "','" + dc.OrgNr + "')";
                                SqlCommand insertDaughterCompanyCommand = new SqlCommand(insertDaughterCompanyString, connection);
                                insertDaughterCompanyCommand.ExecuteNonQuery();
                            }
                        }
                    }
                }
                catch (InvalidOperationException ioe)
                {
                    Console.Error.WriteLine("Failed to perform operation: " + ioe.Message);
                }
                catch (System.Data.SqlClient.SqlException ex)
                {
                    Console.Error.WriteLine("Failed to connect to data source: " + ex.Message);
                }
                finally
                {
                    connection.Close();
                }
            }
        }
    }
}
